@@ -39,7 +39,7 @@ int open_TCP_socket(struct peer *peer, const char *hostname);
 struct addrinfo * find_best_addr(struct addrinfo *answer);
 bool connect_TCP_socket(int sockfd, struct addrinfo *addr);
 bool nts_set_cert_search(SSL_CTX *ctx, const char *filename);
-void set_hostname(SSL *ssl, struct peer *peer, const char *hostname);
+void set_hostname(SSL *ssl, const char *hostname);
 bool check_certificate(SSL *ssl, struct peer *peer);
 bool check_alpn(SSL *ssl, struct peer *peer, const char *hostname);
 bool nts_client_send_request(SSL *ssl, struct peer *peer);
@@ -123,7 +123,7 @@ bool nts_probe(struct peer * peer) {
 		ssl = SSL_new(ctx);
 		SSL_CTX_free(ctx);
 	}
-	set_hostname(ssl, peer, hostname);
+	set_hostname(ssl, hostname);
 	SSL_set_fd(ssl, server);
 
 	if (1 != SSL_connect(ssl)) {
@@ -316,7 +316,7 @@ int open_TCP_socket(struct peer *peer, const char *hostname) {
 
 struct addrinfo *find_best_addr(struct addrinfo *answer) {
 	/* default to first one */
-        return(answer);
+	return(answer);
 }
 
 
@@ -327,7 +327,7 @@ struct addrinfo *find_best_addr(struct addrinfo *answer) {
  */
 bool connect_TCP_socket(int sockfd, struct addrinfo *addr) {
 	char errbuf[100];
-        int err;
+	int err;
 	fd_set fdset;
 	struct timeval timeout;
 	int so_error;
@@ -369,7 +369,7 @@ bool connect_TCP_socket(int sockfd, struct addrinfo *addr) {
 		return false;
 	}
 
-        if (0 != so_error) {
+	if (0 != so_error) {
 		ntp_strerror_r(so_error, errbuf, sizeof(errbuf));
 		msyslog(LOG_INFO, "NTSc: connect_TCP_socket: connect failed: %s", errbuf);
 		return false;
@@ -386,7 +386,7 @@ bool connect_TCP_socket(int sockfd, struct addrinfo *addr) {
 }
 
 
-void set_hostname(SSL *ssl, struct peer *peer, const char *hostname) {
+void set_hostname(SSL *ssl, const char *hostname) {
 	char host[256], *tmp;
 
 	/* chop off trailing :port */
@@ -400,9 +400,12 @@ void set_hostname(SSL *ssl, struct peer *peer, const char *hostname) {
 		*tmp = 0;
 	}
 
-// https://wiki.openssl.org/index.php/Hostname_validation
-	UNUSED_ARG(peer);
-	SSL_set_hostflags(ssl, X509_CHECK_FLAG_NO_WILDCARDS);
+/* https://wiki.openssl.org/index.php/Hostname_validation
+ * draft-ietf-uta-rfc6125bis section 3 relaxes the restrictions around the use
+ * of wildcards to make it clear that they're permitted unless specifically
+ * prohibited in an RFC
+ */
+	SSL_set_hostflags(ssl, X509_CHECK_FLAG_NO_PARTIAL_WILDCARDS);
 	SSL_set1_host(ssl, host);
 	msyslog(LOG_DEBUG, "NTSc: set cert host: %s", host);
 
@@ -410,35 +413,69 @@ void set_hostname(SSL *ssl, struct peer *peer, const char *hostname) {
 
 bool check_certificate(SSL *ssl, struct peer* peer) {
 	X509 *cert = SSL_get_peer_certificate(ssl);
+	X509_NAME *certname;
+	GENERAL_NAMES *gens;
+	char name[200];
+	int certok;
+	int numgens = 0;
 
 	if (NULL == cert) {
 		msyslog(LOG_INFO, "NTSc: No certificate");
 		if (!(FLAG_NTS_NOVAL & peer->cfg.flags))
 			return false;
 		return true;
+	}
+
+	certname = X509_get_subject_name(cert);
+	X509_NAME_oneline(certname, name, sizeof(name));
+	msyslog(LOG_INFO, "NTSc: certificate subject name: %s", name);
+	certname = X509_get_issuer_name(cert);
+	X509_NAME_oneline(certname, name, sizeof(name));
+	msyslog(LOG_INFO, "NTSc: certificate issuer name: %s", name);
+	/* print SAN:DNS strings */
+	gens = X509_get_ext_d2i(cert, NID_subject_alt_name, 0, 0);
+	if (gens) {
+	  char buff[150];
+	  numgens = sk_GENERAL_NAME_num(gens);
+	  buff[0] = 0;
+	  for (int i = 0; i<numgens; i++) {
+	    const GENERAL_NAME *gen;
+	    const char *dnsname;
+	    unsigned int len;
+	    gen = sk_GENERAL_NAME_value(gens, i);
+	    if (gen->type != GEN_DNS)
+	      continue;
+	    // string is NUL terminated but may have internal NULs
+	    len = (unsigned int)ASN1_STRING_length(gen->d.ia5);
+	    dnsname = (const char *)ASN1_STRING_get0_data(gen->d.ia5);
+	    if (0 != buff[0])
+	      strlcat(buff, ", ", sizeof(buff));
+	    strlcat(buff, dnsname, sizeof(buff));
+	    if (len != strlen(dnsname))
+	      strlcat(buff, "??", sizeof(buff));
+	  }
+	  msyslog(LOG_INFO, "NTSc: SAN:DNS %s", buff);
+	  GENERAL_NAMES_free(gens);
+	}
+	if (0 == numgens) {
+	  const char *peername = SSL_get0_peername(ssl);
+	  msyslog(LOG_INFO, "NTSc: matching with subject:CN %s", peername);
+	} else if (1 > numgens) {
+	  const char *peername = SSL_get0_peername(ssl);
+	  msyslog(LOG_INFO, "NTSc: matching with SAN:DNS: %s", peername);
+	}
+	X509_free(cert);
+	certok = SSL_get_verify_result(ssl);
+	if (X509_V_OK == certok) {
+		msyslog(LOG_INFO, "NTSc: certificate is valid.");
 	} else {
-		X509_NAME *certname;
-		char name[200];
-		int certok;
-		certname = X509_get_subject_name(cert);
-		X509_NAME_oneline(certname, name, sizeof(name));
-		msyslog(LOG_INFO, "NTSc: certificate subject name: %s", name);
-		certname = X509_get_issuer_name(cert);
-		X509_NAME_oneline(certname, name, sizeof(name));
-		msyslog(LOG_INFO, "NTSc: certificate issuer name: %s", name);
-		X509_free(cert);
-		certok = SSL_get_verify_result(ssl);
-		if (X509_V_OK == certok) {
-			msyslog(LOG_INFO, "NTSc: certificate is valid.");
-		} else {
-			msyslog(LOG_ERR, "NTSc: certificate invalid: %d=>%s",
-				certok, X509_verify_cert_error_string(certok));
-			if (FLAG_NTS_NOVAL & peer->cfg.flags) {
-				msyslog(LOG_INFO, "NTSc: noval - accepting invalid cert.");
-				return true;
-			}
-			return false;
+		msyslog(LOG_ERR, "NTSc: certificate invalid: %d=>%s",
+			certok, X509_verify_cert_error_string(certok));
+		if (FLAG_NTS_NOVAL & peer->cfg.flags) {
+			msyslog(LOG_INFO, "NTSc: noval - accepting invalid cert.");
+			return true;
 		}
+		return false;
 	}
 	return true;
 }
@@ -579,7 +616,7 @@ bool nts_client_process_response_core(uint8_t *buff, int transferred, struct pee
 
 	buf.next = buff;
 	buf.left = transferred;
-	while (buf.left > 0) {
+	while (buf.left >= NTS_KE_HDR_LNG) {
 		uint16_t type, data, port;
 		bool critical = false;
 		int length, keylength;
@@ -692,6 +729,9 @@ bool nts_client_process_response_core(uint8_t *buff, int transferred, struct pee
 		} /* case */
 	}   /* while */
 
+	if (buf.left > 0)
+		return false;
+
 	if (NO_AEAD == peer->nts_state.aead) {
 		msyslog(LOG_ERR, "NTSc: No AEAD algorithim.");
 		return false;
