@@ -1467,7 +1467,17 @@ class TestControlSession(unittest.TestCase):
     def test___parse_varlist(self):
         # Init
         cls = self.target()
-        cls.response = 'srcadr=0.0.0.0, srcport=0, srchost="0.ubuntu.pool.ntp.org",\r\ndstadr=0.0.0.0, dstport=0, leap=3, stratum=16, precision=-22,\r\nrootdelay=0.000, rootdisp=0.000, refid=POOL,\r\nreftime=0x00000000.00000000, rec=0x00000000.00000000, reach=0x0,\r\nunreach=0, hmode=3, pmode=0, hpoll=6, ppoll=10, headway=0, flash=0x1600,\r\nkeyid=0, offset=0.000, delay=0.000, dispersion=16000.000, jitter=0.000,\r\nfiltdelay= 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00,\r\nfiltoffset= 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00,\r\nfiltdisp= 16000.00 16000.00 16000.00 16000.00 16000.00 16000.00 16000.00 16000.00,\r\nnovalue, blankvalue=, quotedvalue="jabber"'
+        cls.response = (
+            'srcadr=0.0.0.0, srcport=0, srchost="0.ubuntu.pool.ntp.org",\r\n'
+            'dstadr=0.0.0.0, dstport=0, leap=3, stratum=16, precision=-22,\r\n'
+            'rootdelay=0.000, rootdisp=0.000, refid=POOL,\r\n'
+            'reftime=0x00000000.00000000, rec=0x00000000.00000000, reach=0x0,\r\n'
+            'unreach=0, hmode=3, pmode=0, hpoll=6, ppoll=10, headway=0, flash=0x1600,\r\n'
+            'keyid=0, offset=0.000, delay=0.000, dispersion=16000.000, jitter=0.000,\r\n'
+            'filtdelay= 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00,\r\n'
+            'filtoffset= 0.00 0.00 0.00 0.00 0.00 0.00 0.00 0.00,\r\n'
+            'filtdisp= 16000.00 16000.00 16000.00 16000.00 16000.00 16000.00 16000.00 16000.00,\r\n'
+            'novalue, blankvalue=, quotedvalue="jabber"')
         # Test with basic packet
         self.assertEqual(cls._ControlSession__parse_varlist(),
                          odict((("srcadr", "0.0.0.0"), ("srcport", 0),
