@@ -12,6 +12,8 @@ def options_cmd(ctx, config):
                    default=False, help="Enable debugging code")
     grp.add_option('--enable-debug-gdb', action='store_true',
                    default=False, help="Enable GDB debugging symbols")
+    grp.add_option('--enable-attic', action='store_true',
+                   default=False, help="Enable building attic/*.")
     grp.add_option('--disable-nts', action='store_true',
                    default=False, help="Disable NTS.")
     grp.add_option('--disable-droproot', action='store_true',
